<?php
require_once 'WsResponseFactory.class.php';
require_once 'Auth.class.php';
require_once 'Pz2Settings.class.php';
require_once 'CategoryTreeHash.class.php';

class Pz2Proxy
{
    private $response;
    private $request;
    private $session;
    private $conf;

    public function __construct (&$r, &$s )
    {
        $this->request = &$r;
        $this->session = &$s;
        $this->loadConf(MASTERKEY_AUTH_CONF);

        isset( $this->request['output'] )
            ? $output = $this->request['output']
            : $output = 'json';

        $this->response = 
            WsResponseFactory::getResponse($output);
    }

    public function start ()
    {
        $this->response->header();
        $this->dispatchCommand();
    }

    public function sendResponse ()
    {
        echo $this->response->printResponse();
    }

    private function dispatchCommand ()
    {
        switch( strtolower( $this->request['command'] ) )
        {
            case 'init':
                $this->response->setRoot('init');
                $this->doInit();
                break;

            default:
                $this->response->setRoot('pzproxy');
                $this->response->addError('Command unknown or missing.');
        }
    }

    private function doInit()
    {
        $auth = new Auth();
        if ( isset( $this->session['auth']['policy'] ) 
            && $auth->authByPolicy( $this->session['auth']['policy'] ) )
        {
            $targets = $auth->authorize();
            $settings = new Pz2Settings($this->conf['host'] . ':' . $this->conf['port']);
            $settings->initPz2();
            $settings->populate( $targets );
            
            try {
                $settings->send();
            } catch (Exception $e) {
                $this->response->addError($e->getMessage());
            }
            
            $catTree = new CategoryTreeHash();
            $catTree->insertTargets($targets);
            $target_list = array();

            foreach ($targets as $t) {
                $target = $t->getTarget();
                $target_list[$target->getConnString()] = array(
                    
                        'target_name'   => $target->getTargetName(),
                        'rec_enc'       => $target->getRecEnc(),
                        'query_enc'     => $target->getQueryEnc(),
                        'rec_syn'       => $target->getReqSyn(),
                        'holding_type'  => $target->getHoldingType(),
                        'holding_xsl'   => $target->getHoldingXsl()

                    );
            }
            
            $this->response->addEntry('logo', $auth->getGroupLogoURI() );
            $this->response->addEntry('targets', $target_list);
            $this->response->addEntry('sessionid',  $settings->getSessionId() );
            $this->response->addEntry('categories', $catTree->getTree() );
            
        } else {
            $this->response->addError('Not authenticated.');
        }

    }


    private function loadConf ($pzproxy_conf)
    {
        if ( file_exists($pzproxy_conf) ) {
            $auth_conf = new DOMDocument();
            $auth_conf->load($pzproxy_conf);
            $this->conf = array();
            $this->conf['host'] = $auth_conf->getElementsByTagName('pz2_host')->item(0)->nodeValue;
            $this->conf['port'] = $auth_conf->getElementsByTagName('pz2_port')->item(0)->nodeValue;
        } else {
            throw new Exception('Please add the file: ' . $pzproxy_conf);
        }
    }
}
