<?php

/* $Id: Pz2Settings.class.php,v 1.7 2007/07/27 09:56:30 jakub Exp $
 * -----------------------------------------------------------------------
 * Pz2 dynamic configuration class
 */


class Pz2Settings
{
    private $url;
    private $options;
    private $optionStrArr = array();
    private $sessionId;
    private $maxLength = 4050;


    public function __construct ($url = '')
    {
        if ($url != '')
            $this->url = $url;
    }

    public function setOption ($targetid, $key, $value)
    {
        if ( !isset($this->options[$targetid]) )
            $this->options[$targetid] = array();

        $this->options[$targetid][$key] = $value;
    }

    public function setSessionId ($id)
    {
        $this->sessionId = $id;
    }

    public function getSessionId ()
    {
        return $this->sessionId;
    }

    public function setUrl ($url)
    {
        $this->url = $url;
    }

    public function getUrl ()
    {
        return $this->url;
    }

    public function setMaxLength ($length)
    {
        $this->maxLength = $length;
    }

    public function populate ($zTargetCollection)
    {
       foreach ($zTargetCollection as $entry ) {
            $target = $entry->getTarget();
            $zuser = $entry->getZUser();
            $zpass = $entry->getZPass();
            $targetId = urlencode($target->getConnString());
            $url_recipe = $target->getUrlRecipe();
            
            $this->setOption($targetId, 'pz:name', $target->getTargetName());
            //$this->setOption($targetId, 'pz:id', $targetId);            
            $this->setOption($targetId, 'pz:nativesyntax', $target->getRecEnc());
            $this->setOption($targetId, 'pz:queryencoding', $target->getQueryEnc());
            $this->setOption($targetId, 'pz:requestsyntax', $target->getReqSyn());

            $xslt_trans = $target->getXslt();
            if ( !isset( $xslt_trans ) || $xslt_trans == '' )
                $xslt_trans = "auto";
            
            $this->setOption($targetId, 'pz:xslt', $xslt_trans );

            if ( $zuser && $zpass ) {
                $this->setOption($targetId, 'pz:authentication', 
                                    $zuser.'/'.$zpass );
            }
            else if ( $zuser ) {
                $this->setOption($targetId, 'pz:authentication',
                                    $zuser );
            }

            if ($url_recipe)
                $this->setOption($targetId, 'url_recipe',
                                    $url_recipe );

            $cclmap_id = $target->getCclMapId();
            $c = new Criteria();
            $c->add(CclSpecPeer::CCL_MAP_ID, $cclmap_id);
            $cclspecs = CclSpecPeer::doSelect($c);

            foreach ($cclspecs as $ccl) {
                $this->setOption($targetId,
                                     'pz:cclmap:'. $ccl->getCclQual(),
                                     $ccl->getPqf());
            }
        }
    }

    public function initPz2 ()
    {
        $ch = curl_init();
        curl_setopt( $ch, CURLOPT_URL, $this->url.'/search.pz2?command=init&clear=1' );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
        curl_setopt( $ch, CURLOPT_HEADER, false );

        $response = curl_exec($ch);

        if ( $response ) {
            $doc = new DOMDocument();
            $doc->loadXML( curl_exec($ch) );
            $this->sessionId = 
                $doc->getElementsByTagName('session')->item(0)->nodeValue;
        } else {
            throw new Exception('Pazpar2 init failed! ('.$this->url.')');
        }
    }

    public function send ()
    {
        if ( !isset($this->url) ) {
            throw new Exception('URL is not specified');
        }

        $this->encode();

        foreach ( $this->optionStrArr as $optStr ) {
            $ch = curl_init();
            curl_setopt( $ch, CURLOPT_URL, $this->url.$optStr );
            curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
            curl_setopt( $ch, CURLOPT_HEADER, false );
            $this->checkResponse(curl_exec($ch));
        }
    }

    public function toString()
    {
        $this->encode();
        return $this->optionStrArr[0];
    }

    private function encode ()
    {
        if ( !isset($this->sessionId) ) {
            throw new Exception('SESSIONID is not specified');
        }

        if ( !isset($this->options) ) {
            throw new Exception('Settings are empty.');
        }

        $command = '/search.pz2?command=settings&session='.
                           $this->sessionId;
        $optStrNum = 0;

        $this->optionStrArr[$optStrNum] = $command;

        foreach ($this->options as $target => $settings) {
            $targetSetting = '';
            
            foreach ($settings as $key => $value) {
                $targetSetting .= '&' . $key . '[' . $target . ']=' . urlencode($value);
            }
            
            if ( (strlen($this->url) 
                + strlen($this->optionStrArr[$optStrNum]) 
                + strlen($targetSetting)) 
                < $this->maxLength ) {
                $this->optionStrArr[$optStrNum] .= $targetSetting;
            } else {
                $optStrNum++;
                $this->optionStrArr[$optStrNum] = $command.$targetSetting;
            }
        }
    }

    private function checkResponse ($response)
    {
        $doc = new DOMDocument();
        $doc->loadXML($response);

        if ( isset($doc->getElementsByTagName('status')->item(0)->nodeValue)
            && $doc->getElementsByTagName('status')->item(0)->nodeValue == 'OK' )
            return;
        else
            throw new Exception('Settings not applied properly');
    }

}
